@@ -1,6 +1,6 @@
 'Scrypt for Python'
 
-__version__ = '1.0.3'
+__version__ = '1.1.0'
 
 # First, try loading libscrypt
 _done = False
@@ -20,6 +20,17 @@ if not _done:
     else:
         _done = True
 
+# Unless we are on pypy, we want to try libsodium as well
+if not _done:
+    import platform
+    if platform.python_implementation() != 'PyPy':
+        try:
+            from pylibsodium import *
+        except ImportError:
+            pass
+        else:
+            _done = True
+
 # If that didn't work either, the inlined Python version
 if not _done:
     try:
