@@ -35,11 +35,10 @@
 
 
 import hashlib, hmac
+import numbers
 import struct
 
-
 import mcf as mcf_mod
-
 from consts import *
 
 
@@ -80,21 +79,21 @@ def scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64):
 
 
     def integerify(B, r):
-        '''"A bijection from ({0, 1} ** k) to {0, ..., (2 ** k) - 1"'''
+        """A bijection from ({0, 1} ** k) to {0, ..., (2 ** k) - 1"""
 
         Bi = (2 * r - 1) * 16
         return B[Bi]
 
 
     def R(X, destination, a1, a2, b):
-        '''A single Salsa20 row operation'''
+        """A single Salsa20 row operation"""
 
         a = (X[a1] + X[a2]) & 0xffffffff
         X[destination] ^= ((a << b) | (a >> (32 - b)))
 
 
     def salsa20_8(B, x, src, s_start, dest, d_start):
-        '''Salsa20/8 http://en.wikipedia.org/wiki/Salsa20'''
+        """Salsa20/8 http://en.wikipedia.org/wiki/Salsa20"""
 
         # Merged blockxor for speed
         for i in xrange(16):
@@ -118,7 +117,7 @@ def scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64):
 
 
     def blockmix_salsa8(BY, Yi, r):
-        '''Blockmix; Used by SMix'''
+        """Blockmix; Used by SMix"""
 
         start = (2 * r - 1) * 16
         X = BY[start:start+16]                             # BlockMix - 1
@@ -135,7 +134,7 @@ def scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64):
 
 
     def smix(B, Bi, r, N, V, X):
-        '''SMix; a specific case of ROMix based on Salsa20/8'''
+        """SMix; a specific case of ROMix based on Salsa20/8"""
 
         array_overwrite(B, Bi, X, 0, 32 * r)               # ROMix - 1
 
@@ -150,9 +149,17 @@ def scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64):
 
         array_overwrite(X, 0, B, Bi, 32 * r)               # ROMix - 10
 
-
+    if not isinstance(password, bytes):
+        raise TypeError('password must be a byte string')
     if not isinstance(salt, bytes):
-        raise TypeError('scrypt salt must be a byte string')
+        raise TypeError('salt must be a byte string')
+    if not isinstance(N, numbers.Integral):
+        raise TypeError('N must be an integer')
+    if not isinstance(r, numbers.Integral):
+        raise TypeError('r must be an integer')
+    if not isinstance(p, numbers.Integral):
+        raise TypeError('p must be an integer')
+
     if N < 2 or (N & (N - 1)):
         raise ValueError('scrypt N must be a power of 2 greater than 1')
     if N > 2 ** 63:
@@ -162,12 +169,14 @@ def scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64):
     if p <= 0:
         raise ValueError('scrypt p must be positive')
 
-    B  = _pbkdf2('sha256', password, salt, 1, p * 128 * r)
-
     # Everything is lists of 32-bit uints for all but pbkdf2
-    B  = list(struct.unpack('<%dI' % (len(B) // 4), B))
-    XY = [0] * (64 * r)
-    V  = [0] * (32 * r * N)
+    try:
+        B  = _pbkdf2('sha256', password, salt, 1, p * 128 * r)
+        B  = list(struct.unpack('<%dI' % (len(B) // 4), B))
+        XY = [0] * (64 * r)
+        V  = [0] * (32 * r * N)
+    except (MemoryError, OverflowError):
+        raise ValueError("scrypt parameters don't fit in memory")
 
     for i in xrange(p):
         smix(B, i * 32 * r, r, N, V, XY)
@@ -195,7 +204,7 @@ def scrypt_mcf_check(mcf, password):
 
 
 if __name__ == "__main__":
+    import sys
     import tests
-    print('Testing scrypt...')
-    tests.run_tests(scrypt, scrypt_mcf, scrypt_mcf_check)
+    tests.run_scrypt_suite(sys.modules[__name__])
 
